/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.sync.base

import android.util.Log

abstract class BaseIncrementalSyncHandler<Context, Entry, Key> :
  BaseSyncHandler<Context, Entry, Key>(),
  IncrementalSyncHandler<Context> {
  abstract suspend fun loadLatestEntry(context: Context): Entry?
  abstract suspend fun loadAdded(context: Context, latest: Entry): List<Entry>

  override suspend fun syncIncremental(context: Context) {
    if (syncState.compareAndSet(State.Idle, State.Loading) ||
      syncState.compareAndSet(State.Error(), State.Loading)) {
      Log.w(this::class.simpleName, "Started incremental sync")
      val success = try {
        val result = withTransaction {
          val latestEntry = loadLatestEntry(context)
          if (latestEntry != null) {
            val addedEntries = loadAdded(context, latestEntry)
            for (loadedEntry in addedEntries) {
              store(loadedEntry)
            }
            true
          } else {
            false
          }
        }
        syncState.value = State.Idle
        Log.w(this::class.simpleName, "Finished incremental sync")
        result
      } catch (e: Exception) {
        Log.e(this::class.simpleName, "Error while syncing data", e)
        syncState.value = State.Error("Error while syncing data: $e")
        false
      }
      // If we can't do an incremental sync, do a full sync
      if (!success) {
        sync(context)
      }
    } else {
      Log.w(this::class.simpleName, "Already syncing, disregarding sync request")
    }
  }
}
