/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui

import androidx.compose.animation.EnterTransition
import androidx.compose.animation.ExitTransition
import androidx.compose.foundation.layout.Box
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SnackbarDuration
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navArgument
import de.chaosdorf.mete.model.UserId
import de.chaosdorf.meteroid.model.ServerId
import de.chaosdorf.meteroid.ui.drinks.DrinkListScreen
import de.chaosdorf.meteroid.ui.drinks.DrinkListViewModel
import de.chaosdorf.meteroid.ui.money.MoneyListScreen
import de.chaosdorf.meteroid.ui.money.MoneyListViewModel
import de.chaosdorf.meteroid.ui.navigation.MeteroidBottomBar
import de.chaosdorf.meteroid.ui.navigation.MeteroidTopBar
import de.chaosdorf.meteroid.ui.navigation.Routes
import de.chaosdorf.meteroid.ui.servers.AddServerScreen
import de.chaosdorf.meteroid.ui.servers.AddServerViewModel
import de.chaosdorf.meteroid.ui.servers.ServerListScreen
import de.chaosdorf.meteroid.ui.servers.ServerListViewModel
import de.chaosdorf.meteroid.ui.transactions.TransactionListScreen
import de.chaosdorf.meteroid.ui.transactions.TransactionViewModel
import de.chaosdorf.meteroid.ui.users.UserListScreen
import de.chaosdorf.meteroid.ui.users.UserListViewModel
import de.chaosdorf.meteroid.ui.wrapped.WrappedScreen
import de.chaosdorf.meteroid.ui.wrapped.WrappedViewModel
import de.chaosdorf.meteroid.util.popUpToRoot

@Composable
fun AppRouter(viewModel: AppViewModel) {
  val navController = rememberNavController()
  val snackbarHostState = remember { SnackbarHostState() }
  val navigationViewModel = hiltViewModel<NavigationViewModel>()

  val initialBackStack by viewModel.initialBackStack.collectAsState()
  val offline by viewModel.offline.collectAsState()

  navController.addOnDestinationChangedListener { _, _, arguments ->
    val serverId = arguments?.getLong("server")?.let(::ServerId)
    val userId = arguments?.getLong("user")?.let(::UserId)

    navigationViewModel.serverId.value = serverId
    navigationViewModel.userId.value = userId
  }

  LaunchedEffect(initialBackStack) {
    initialBackStack?.let { initialBackStack ->
      navController.popUpToRoot()
      for (entry in initialBackStack) {
        navController.navigate(entry)
      }
    }
  }

  LaunchedEffect(offline) {
    snackbarHostState.currentSnackbarData?.dismiss()
    if (offline) {
      snackbarHostState.showSnackbar(
        message = "Unable to connect to server",
        duration = SnackbarDuration.Indefinite
      )
    }
  }

  Box {
    Scaffold(
      topBar = {
        MeteroidTopBar(navController, navigationViewModel, Modifier.align(Alignment.TopCenter))
      },
      bottomBar = {
        MeteroidBottomBar(navController, navigationViewModel)
      },
      snackbarHost = {
        SnackbarHost(hostState = snackbarHostState)
      }
    ) { paddingValues ->
      NavHost(
        navController,
        startDestination = Routes.Servers.List,
        enterTransition = { EnterTransition.None },
        exitTransition = { ExitTransition.None },
        popEnterTransition = { EnterTransition.None },
        popExitTransition = { ExitTransition.None }
      ) {
        composable(Routes.Servers.Add) { _ ->
          val hiltViewModel = hiltViewModel<AddServerViewModel>()
          AddServerScreen(navController, hiltViewModel, paddingValues)
        }
        composable(Routes.Servers.List) { _ ->
          val hiltViewModel = hiltViewModel<ServerListViewModel>()
          ServerListScreen(navController, hiltViewModel, paddingValues)
        }
        /*
      composable(Routes.Users.Add) { _ ->
        AddUserScreen(
          hiltViewModel(),
          onAdd = { navController.navigate(Routes.Users.List) }
        )
      }
       */
        composable(
          Routes.Users.List,
          arguments = listOf(
            navArgument("server") { type = NavType.LongType }
          )
        ) { _ ->
          val hiltViewModel = hiltViewModel<UserListViewModel>()
          UserListScreen(navController, hiltViewModel, paddingValues)
        }
        composable(
          Routes.Home.Purchase,
          arguments = listOf(
            navArgument("server") { type = NavType.LongType },
            navArgument("user") { type = NavType.LongType },
          )
        ) { _ ->
          val hiltViewModel = hiltViewModel<DrinkListViewModel>()
          DrinkListScreen(navController, hiltViewModel, paddingValues)
        }
        composable(
          Routes.Home.Deposit,
          arguments = listOf(
            navArgument("server") { type = NavType.LongType },
            navArgument("user") { type = NavType.LongType },
          )
        ) { _ ->
          val hiltViewModel = hiltViewModel<MoneyListViewModel>()
          MoneyListScreen(navController, hiltViewModel, paddingValues)
        }
        composable(
          Routes.Home.History,
          arguments = listOf(
            navArgument("server") { type = NavType.LongType },
            navArgument("user") { type = NavType.LongType },
          )
        ) { _ ->
          val hiltViewModel = hiltViewModel<TransactionViewModel>()
          TransactionListScreen(hiltViewModel, paddingValues)
        }
        composable(
          Routes.Home.Wrapped,
          arguments = listOf(
            navArgument("server") { type = NavType.LongType },
            navArgument("user") { type = NavType.LongType },
          )
        ) { _ ->
          val hiltViewModel = hiltViewModel<WrappedViewModel>()
          WrappedScreen(hiltViewModel, paddingValues)
        }
      }
    }
  }
}
