/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import de.chaosdorf.mete.model.UserId
import de.chaosdorf.meteroid.model.PinnedUserRepository
import de.chaosdorf.meteroid.model.ServerId
import de.chaosdorf.meteroid.model.ServerRepository
import de.chaosdorf.meteroid.model.UserRepository
import de.chaosdorf.meteroid.sync.AccountProvider
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.flatMapLatest
import kotlinx.coroutines.flow.flowOf
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class NavigationViewModel @Inject constructor(
  serverRepository: ServerRepository,
  userRepository: UserRepository,
  pinnedUserRepository: PinnedUserRepository,
  private val accountProvider: AccountProvider
) : ViewModel() {
  val serverId = MutableStateFlow<ServerId?>(null)
  val userId = MutableStateFlow<UserId?>(null)

  val server = serverId.flatMapLatest {
    if (it == null) flowOf(null)
    else serverRepository.getFlow(it)
  }.stateIn(viewModelScope, SharingStarted.WhileSubscribed(), null)

  val user = combine(serverId, userId) { serverId, userId ->
    if (serverId == null || userId == null) null
    else Pair(serverId, userId)
  }.flatMapLatest {
    if (it == null) flowOf(null)
    else userRepository.getFlow(it.first, it.second)
  }.stateIn(viewModelScope, SharingStarted.WhileSubscribed(), null)

  val pinned = combine(serverId, userId) { serverId, userId ->
    if (serverId == null || userId == null) null
    else Pair(serverId, userId)
  }.flatMapLatest {
    if (it == null) flowOf(null)
    else pinnedUserRepository.isPinnedFlow(it.first, it.second)
  }.stateIn(viewModelScope, SharingStarted.WhileSubscribed(), null)

  fun togglePin(serverId: ServerId, userId: UserId) {
    viewModelScope.launch {
      accountProvider.togglePin(serverId, userId)
    }
  }
}
