/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui.navigation

import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.NavigationBar
import androidx.compose.material3.NavigationBarItem
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.navigation.NavController
import androidx.navigation.compose.currentBackStackEntryAsState
import de.chaosdorf.meteroid.ui.NavigationViewModel
import de.chaosdorf.meteroid.ui.theme.onPrimaryContainerTinted
import de.chaosdorf.meteroid.util.popUpToRoot
import kotlinx.datetime.Clock
import kotlinx.datetime.TimeZone
import kotlinx.datetime.toLocalDateTime
import java.time.Month

@Composable
fun MeteroidBottomBar(
  navController: NavController,
  viewModel: NavigationViewModel
) {
  val user by viewModel.user.collectAsState()
  val historyEnabled = user?.audit == true
  val wrappedEnabled = Clock.System.now()
    .toLocalDateTime(TimeZone.currentSystemDefault())
    .month.let { it == Month.NOVEMBER || it == Month.DECEMBER }
  val navBackStackEntry by navController.currentBackStackEntryAsState()
  val currentDestination = navBackStackEntry?.destination
  val activeRoute = if (user == null) null
  else HomeSections.entries.find {
    it.route == currentDestination?.route
  }

  if (activeRoute != null) {
    NavigationBar(
      contentColor = MaterialTheme.colorScheme.onPrimaryContainerTinted
    ) {
      for (route in HomeSections.entries) {
        if (wrappedEnabled || route != HomeSections.WRAPPED) {
          NavigationBarItem(
            icon = {
              Icon(
                if (route == activeRoute) route.iconActive else route.icon,
                contentDescription = route.title,
                tint = MaterialTheme.colorScheme.onPrimaryContainerTinted
              )
            },
            label = { Text(route.title, color = MaterialTheme.colorScheme.onPrimaryContainerTinted) },
            selected = route == activeRoute,
            onClick = {
              navController.popUpToRoot()
              navController.navigate(Routes.Servers.List)
              navController.navigate(Routes.Users.list(user!!.serverId))
              navController.navigate(
                route.withArguments(
                  "server" to user!!.serverId.value.toString(),
                  "user" to user!!.userId.value.toString()
                )
              )
            },
            enabled = when (route) {
              HomeSections.PURCHASE,
              HomeSections.DEPOSIT -> true

              HomeSections.HISTORY,
              HomeSections.WRAPPED -> historyEnabled
            }
          )
        }
      }
    }
  }
}
