/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui.navigation

import android.util.Log
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.PushPin
import androidx.compose.material.icons.outlined.PushPin
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import de.chaosdorf.meteroid.R
import de.chaosdorf.meteroid.ui.NavigationViewModel
import de.chaosdorf.meteroid.ui.PriceBadge
import de.chaosdorf.meteroid.util.rememberAvatarPainter
import okhttp3.HttpUrl.Companion.toHttpUrl

@Composable
fun MeteroidTopBar(
  navController: NavController,
  viewModel: NavigationViewModel,
  modifier: Modifier = Modifier
) {
  val server by viewModel.server.collectAsState()
  val user by viewModel.user.collectAsState()
  val pinned by viewModel.pinned.collectAsState()

  val backstack by navController.currentBackStack.collectAsState()
  val canNavigateUp = backstack.size > 2
  LaunchedEffect(backstack) {
    val backstackEntries = backstack.map {
      it.destination.route
        ?.replace("{server}", it.arguments?.getLong("server")?.toString() ?: "{server}")
        ?.replace("{user}", it.arguments?.getLong("user")?.toString() ?: "{user}")
    }
    Log.e("Navigation", "BACKSTACK: [${backstackEntries.joinToString(" › ")}]")
  }

  val avatarPainter = rememberAvatarPainter(
    user?.gravatarUrl,
    32.dp, 32.dp,
    MaterialTheme.colorScheme.primary
  )

  val iconPainter = painterResource(R.drawable.ic_launcher)

  Surface(
    modifier = modifier
      .padding(8.dp)
      .height(64.dp),
    color = MaterialTheme.colorScheme.surface,
    shadowElevation = 6.dp,
    tonalElevation = 6.dp,
    shape = RoundedCornerShape(32.dp),
    onClick = { navController.navigateUp() }
  ) {
    Row(modifier = Modifier.padding(8.dp)) {
      if (user != null) {
        Box(
          modifier = Modifier
            .size(48.dp)
            .clip(CircleShape)
            .background(MaterialTheme.colorScheme.primaryContainer)
        ) {
          Image(
            avatarPainter,
            contentDescription = null,
            contentScale = ContentScale.Crop,
            modifier = Modifier.align(Alignment.Center)
          )
        }
      } else if (canNavigateUp) {
        Icon(
          Icons.AutoMirrored.Default.ArrowBack,
          contentDescription = null,
          tint = MaterialTheme.colorScheme.onSurface.copy(alpha = 0.67f),
          modifier = Modifier
            .padding(8.dp)
            .size(32.dp)
        )
      } else {
        Image(
          iconPainter,
          contentDescription = null,
          contentScale = ContentScale.Crop,
          modifier = Modifier
            .size(48.dp)
            .clip(CircleShape)
        )
      }
      Spacer(Modifier.width(16.dp))
      Column(
        modifier = Modifier
          .align(Alignment.CenterVertically)
          .weight(1.0f, fill = true)
      ) {
        when {
          user != null && server != null -> {
            Text(
              user!!.name,
              fontWeight = FontWeight.SemiBold,
              overflow = TextOverflow.Ellipsis,
              softWrap = false
            )
            Text(
              server!!.url.toHttpUrl().host,
              color = MaterialTheme.colorScheme.onSurface.copy(alpha = 0.67f),
              fontWeight = FontWeight.Normal,
              overflow = TextOverflow.Ellipsis,
              softWrap = false
            )
          }

          server != null && server!!.name != null -> {
            Text(
              server!!.name!!,
              fontWeight = FontWeight.SemiBold,
              overflow = TextOverflow.Ellipsis,
              softWrap = false
            )
            Text(
              server!!.url.toHttpUrl().host,
              color = MaterialTheme.colorScheme.onSurface.copy(alpha = 0.67f),
              fontWeight = FontWeight.Normal,
              overflow = TextOverflow.Ellipsis,
              softWrap = false
            )
          }

          server != null -> {
            Text(
              server!!.url.toHttpUrl().host,
              fontWeight = FontWeight.SemiBold,
              overflow = TextOverflow.Ellipsis,
              softWrap = false
            )
          }

          else -> {
            Text(
              "Meteroid",
              fontWeight = FontWeight.SemiBold,
              overflow = TextOverflow.Ellipsis,
              softWrap = false
            )
          }
        }
      }
      Spacer(Modifier.width(16.dp))
      user?.let { user ->
        IconButton(onClick = {
          viewModel.togglePin(user.serverId, user.userId)
        }) {
          Icon(
            if (pinned == true) Icons.Filled.PushPin
            else Icons.Outlined.PushPin,
            contentDescription = null
          )
        }
        Spacer(Modifier.width(8.dp))
        PriceBadge(
          user.balance,
          modifier = Modifier
            .align(Alignment.CenterVertically)
            .padding(end = 12.dp)
        )
      }
    }
  }
}
